package net.wanji.datacenter.service.dataconvert.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.wanji.datacenter.dao.mapper.*;
import net.wanji.datacenter.kafka.ConsumerHandler;
import net.wanji.datacenter.pojo.dto.LanePeriodicDataDTO;
import net.wanji.datacenter.pojo.po.*;
import net.wanji.datacenter.service.dataconvert.DataProcessService;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/03/10 10:30
 * @description: 车道周期实时数据加工处理服务，优化模块实时监控操作数据
 */
@Slf4j
@Service(value = "JN040001LanePeriodicData")
@RequiredArgsConstructor
public class LanePeriodicDataProcessServiceImpl implements DataProcessService {

    private final ConsumerHandler consumerHandler;
    private final CrossBaseLaneInfoMapper crossBaseLaneInfoMapper;
    private final CrossDirDataRealTimeMapper crossDirDataRealTimeMapper;
    private final CrossTurnDataRealTimeMapper crossTurnDataRealTimeMapper;
    private final CrossLaneDataRealTimeMapper crossLaneDataRealTimeMapper;
    private final CrossDataRealTimeMapper crossDataRealTimeMapper;

    @Override
    public String receive(String topic) {
        return consumerHandler.getTopicMessage(topic);
    }

    @Override
    public void send(String topic, Object convertData) {
        consumerHandler.removeTopicMessage(topic);
    }

    @Override
    public Object convert(String originalData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataPO = null;
        try {
            lanePeriodicDataPO = JSONObject.parseObject(originalData, LanePeriodicDataDTO.class);
        } catch (Exception e) {
            log.error("车道周期实时数据转换异常", e);
            throw new Exception();
        }
        return lanePeriodicDataPO;
    }

    @Override
    public void save(Object convertData) throws Exception {
        LanePeriodicDataDTO lanePeriodicDataDTO = (LanePeriodicDataDTO) convertData;

        int laneNum = lanePeriodicDataDTO.getLaneNum();
        List<CrossDataRealTimePO> crossDataRealTimePOS = new ArrayList<>(laneNum);
        List<CrossDirDataRealTimePO> crossDirDataRealTimePOS = new ArrayList<>();
        List<CrossTurnDataRealtimePO> crossTurnDataRealtimePOS = new ArrayList<>();
        List<CrossLaneDataRealTimePO> crossLaneDataRealTimePOS = new ArrayList<>();

        List<LanePeriodicDataDTO.EventList> eventLists = lanePeriodicDataDTO.getEventList();
        if (CollectionUtils.isEmpty(eventLists)) {
            log.error("当前车道周期实时数据车道列表为空,参数信息：", lanePeriodicDataDTO);
            throw new Exception("当前车道周期实时数据车道列表为空");
        }
        // 通过车道编号获取车道信息
        List<String> LaneIdList = eventLists.stream().map(LanePeriodicDataDTO.EventList::getLaneId).collect(Collectors.toList());
        List<CrossBaseLaneInfoPO> crossBaseLaneInfoPOS = crossBaseLaneInfoMapper.selectBatchIds(LaneIdList);

        for (LanePeriodicDataDTO.EventList laneRealTimeItem : eventLists) {
            String laneId = laneRealTimeItem.getLaneId();
            Date now = new Date();
            for (CrossBaseLaneInfoPO laneInfoPO : crossBaseLaneInfoPOS) {
                String id = laneInfoPO.getId();
                String crossId = laneInfoPO.getCrossId();
                // todo 路口状态
                Double trafficFlow = Double.valueOf(laneRealTimeItem.getTrafficFlow());
                String dir = String.valueOf(laneInfoPO.getDir());
                String turn = String.valueOf(laneInfoPO.getTurn());


                if (StringUtils.endsWithIgnoreCase(laneId, id)) {
                    CrossDataRealTimePO crossDataRealTimePO = new CrossDataRealTimePO();
                    crossDataRealTimePO.setCrossId(crossId);
                    crossDataRealTimePO.setStatus(0000);
                    crossDataRealTimePO.setType(1); // 1常规 2异常
                    crossDataRealTimePO.setStartTime(now);
                    crossDataRealTimePO.setDuration(5); // todo 通过第二批次判断

                    crossDataRealTimePO.setUnbalanceStatus(0);
                    crossDataRealTimePO.setUnbalanceIndex(1.0);
                    crossDataRealTimePO.setUnbalanceDirs(dir);
                    crossDataRealTimePO.setCongestionStatus(0);
                    crossDataRealTimePO.setCongestionIndex(1.0);
                    crossDataRealTimePO.setCongestionDirs(dir);
                    crossDataRealTimePO.setSpilloverStatus(0);
                    crossDataRealTimePO.setSpilloverIndex(1.0);
                    crossDataRealTimePO.setSpilloverDirs(dir);


                    crossDataRealTimePO.setTrafficIndex(trafficFlow);

                    crossDataRealTimePOS.add(crossDataRealTimePO);

                    CrossDirDataRealTimePO crossDirDataRealTimePO = new CrossDirDataRealTimePO();
                    crossDirDataRealTimePO.setCrossId(crossId);
                    crossDirDataRealTimePO.setStatus(0000);
                    crossDirDataRealTimePO.setTrafficIndex(trafficFlow);


                    crossDirDataRealTimePOS.add(crossDirDataRealTimePO);

                    CrossTurnDataRealtimePO crossTurnDataRealtimePO = new CrossTurnDataRealtimePO();
                    crossTurnDataRealtimePO.setCrossId(crossId);
                    crossTurnDataRealtimePO.setStatus(0000);

                    crossTurnDataRealtimePOS.add(crossTurnDataRealtimePO);


                    CrossLaneDataRealTimePO crossLaneDataRealTimePO = new CrossLaneDataRealTimePO();
                    crossLaneDataRealTimePO.setCrossId(crossId);
                    crossLaneDataRealTimePO.setId(id);

                    crossLaneDataRealTimePOS.add(crossLaneDataRealTimePO);

                }

            }
        }


        crossDataRealTimeMapper.insert(crossDataRealTimePOS);
        crossDirDataRealTimeMapper.insert(crossDirDataRealTimePOS);
        crossTurnDataRealTimeMapper.insert(crossTurnDataRealtimePOS);
        crossLaneDataRealTimeMapper.insert(crossLaneDataRealTimePOS);

    }
}
